﻿// This script replaces text in the input file based on regular expressions. 
// The script searches for patterns defined in the find expression, and replaces it according to 
// the regex replace expression. 

// Syntax:
// FindAndReplace.exe 
//                    <search type: text|regex>
//                    <search expression>
//                    <substitution expression>
//                    {inputfilepath} 
//                    {outputfilepath}

// Example:
// This example corrects a typo in the input text file
// FindAndReplace.exe text "mistped" "mistyped" {inputfilepath} {outputfilepath}
//
// This example corrects the date format, changing the - to a /
// FindAndReplace.exe regex "(\d\d)-(\d\d)-(\d\d\d\d)" "$1/$2/$3" {inputfilepath} {outputfilepath}

// The regex expessions should use the standard .Net regex syntax, as described at:
// https://msdn.microsoft.com/en-us/library/az24scfc%28v=vs.110%29.aspx
// For input and output files simply use the default {inputfilepath} and {outputfilepath} attribute offered by the
// memoQ template editor. 

// Before finalizing the memoQ template, run the test feature, to make sure the script works properly.


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Text.RegularExpressions;

namespace FindAndReplace
{
    class Program
    {
        static void Main(string[] args)
        {

            // Querying and verifying the arguments of the command

            if (args.Length != 5)
            {
                Console.WriteLine(@"This script replaces text in the input file based on regular expressions. 
The script searches for patterns defined in the find expression, and replaces it according to the regex replace expression. 

Syntax:
FindAndReplace.exe 
                <search type: text|regex>
                <search expression>
                <substitution expression>
                {inputfilepath} 
                {outputfilepath}

Example:
This example corrects a typo in the input text file
FindAndReplace.exe text ""mistped"" ""mistyped"" {inputfilepath} {outputfilepath}
                
This example corrects the date format, changing the - to a /
FindAndReplace.exe regex ""(\d\d)-(\d\d)-(\d\d\d\d)"" ""$1/$2/$3"" {inputfilepath} {outputfilepath}

The regex expessions should use the standard .Net regex syntax, as described at:
https://msdn.microsoft.com/en-us/library/az24scfc%28v=vs.110%29.aspx
For input and output files simply use the default {inputfilepath} and {outputfilepath} attribute offered by the memoQ template editor. 

Before finalizing the memoQ template, run the test feature, to make sure the script works properly.");

                return;
            }
            string strSearchType = args[0];
            string strSearchExp = args[1];
            string strReplaceExp = args[2];
            string strInputFile = args[3];
            string strOutputFile = args[4];

            if (File.Exists(strOutputFile))
            {
                Console.WriteLine("The output file already exist. Please specify a new one.");
                return;
            }

            if ((strSearchType != "regex" && strSearchType != "text") || strSearchExp.Length == 0 ||
                 !File.Exists(strInputFile))
                return;

            bool isRegex = (strSearchType == "regex");
            removeSpaces(isRegex, strSearchExp, strReplaceExp, strInputFile, strOutputFile);
        }

        private static void removeSpaces(bool isRegex, string strSearchExp, string strReplaceExp, string strInputFile, string strOutputFile)
        {
            string content;
            using (StreamReader input = new StreamReader(strInputFile))
            using (StreamWriter output = new StreamWriter(strOutputFile))
            {
                output.AutoFlush = true;
                content = input.ReadToEnd();
                string strResult = "";
                if (isRegex)
                {
                    // regex based replace
                    Regex rgx = new Regex(strSearchExp);
                    strResult = rgx.Replace(content, strReplaceExp);

                }
                else
                {
                    // text based replace
                    strResult = content.Replace(strSearchExp, strReplaceExp);
                }
                output.Write(strResult.ToString());

            }
        }
    }
}
